==> tmaid-2.1.2.tar.gz: tmaid-2.1.2/src/other.c <==
/*
    Text maid
    copyright (c) 1998-2002 Iwamoto,Kazuki http://www.maid.org/ iwm@maid.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <ctype.h>
#include <stdio.h>
#include <gdk/gdkkeysyms.h>
#include "other.h"


/******************************************************************************
*                                                                             *
* åܥåؿ                                                    *
*                                                                             *
******************************************************************************/
static gint result;
static GtkWidget *dialog;


/* ܥ󤬲줿 */
static void message_box_clicked(GtkWidget *widget,gint number)
{
	result=number;
    gtk_widget_destroy(dialog);
}


/* ESC줿 */
static gboolean message_box_dialog_key_press(GtkWidget *widget,
										GdkEventKey *event,gpointer user_data)
{
	if (event->keyval==GDK_Escape)
		gtk_widget_destroy(widget);
	return FALSE;
}


/*	åܥåɽ
	title,ȥ
	 text,ʸ
	  def,ǥեȤΥܥ
	  ...,ܥΥ٥
	  RET,줿ܥ													*/
gint message_box(gchar *title,gchar *text,gint def,...)
{
	va_list ap;
	gchar *name;
	gint i=0;
	GtkWidget *button,*label;

	va_start(ap,def);
	result=-1;
	dialog=gtk_dialog_new();
	gtk_signal_connect_after(GTK_OBJECT(dialog),"key-press-event",
						GTK_SIGNAL_FUNC(message_box_dialog_key_press),NULL);
	gtk_signal_connect(GTK_OBJECT(dialog),"destroy",gtk_main_quit,NULL);
	/* å */
	gtk_window_set_title(GTK_WINDOW(dialog),title);
	label=gtk_label_new(text);
	gtk_misc_set_padding(GTK_MISC(label),8,8);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),label,TRUE,TRUE,0);
	while ((name=va_arg(ap,gchar *))!=NULL) {
		button=gtk_button_new_with_label(name);
		gtk_signal_connect(GTK_OBJECT(button),"clicked",
											message_box_clicked,(gpointer)i);
		GTK_WIDGET_SET_FLAGS(button,GTK_CAN_DEFAULT);
		if (i==def)
			gtk_widget_grab_focus(button);
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
														button,TRUE,FALSE,8);
		i++;
	}
	va_end(ap);

	gtk_grab_add(dialog);
	gtk_widget_show_all(dialog);
	gtk_main();

	return result;
}


/******************************************************************************
*                                                                             *
* ٥ؿ                                                              *
*                                                                             *
******************************************************************************/
/*	ե¸߳ǧ
	file,ե̾
	 RET,TRUE:¸ߤ,FALSE:¸ߤʤ										*/
gboolean isfile(gchar *file)
{
	FILE *fp;

	if ((fp=fopen(file,"r"))!=NULL) {
		fclose(fp);
		return TRUE;
	}
	return FALSE;
}


/*	Сꤹ
	   scroll,륦å
	     func,ʥؿ
	func_data,ǡ
	      min,Ǿ
	      max,
	     page,ڡ
	      pos,															*/
void set_scroll_bar(GtkWidget *scroll,GtkSignalFunc func,gpointer func_data,
										gint min,gint max,gint page,gint pos)
{
	GtkObject *adjust;

	adjust=gtk_adjustment_new(pos,min,max,1,page,page-1);
	gtk_range_set_adjustment(GTK_RANGE(scroll),GTK_ADJUSTMENT(adjust));
	gtk_range_slider_update(GTK_RANGE(scroll));
	if (func!=NULL)
		gtk_signal_connect(GTK_OBJECT(adjust),"value-changed",func,func_data);
}


/*	WindowƤ򥹥뤹
	widget,å
	    dx,Xΰư
	    dy,Yΰư
	     x,뤹ϰϤαXɸ
	     y,뤹ϰϤαYɸ
	 width,뤹ϰϤ
	height,뤹ϰϤι⤵											*/
void scroll_window(GtkWidget *widget,gint dx,gint dy,
										gint x,gint y,gint width,gint height)
{
	GdkGC *gc;
	GdkRectangle rc;

	if (ABS(dx)>width || ABS(dy)>height) {
		rc.x=x;
		rc.y=y;
		rc.width=width;
		rc.height=height;
		gtk_widget_draw(widget,&rc);
	} else if (dx!=0 || dy!=0) {
		gc=gdk_gc_new(widget->window);
		gdk_window_copy_area(widget->window,gc,dx>0?x+dx:x,dy>0?y+dy:y,
										widget->window,dx>0?x:x-dx,dy>0?y:y-dy,
												width-ABS(dx),height-ABS(dy));
		gdk_gc_destroy(gc);
		if (dx!=0) {
			rc.x=dx>0?x:x+width+dx;
			rc.y=y;
			rc.width=ABS(dx);
			rc.height=height;
			gtk_widget_draw(widget,&rc);
		}
		if (dy!=0) {
			rc.x=x;
			rc.y=dy>0?y:y+height+dy;
			rc.width=width;
			rc.height=ABS(dy);
			gtk_widget_draw(widget,&rc);
		}
	}
}


static gchar hex[16]={	'0','1','2','3','4','5','6','7',
						'8','9','A','B','C','D','E','F'};


/*	͢ʸ
	value,
	radix,
	 wide,
	 flag,TRUE:椢,FALSE:ʤ
	  RET,ʸ															*/
gchar *valstr(gint value,gint radix,gint wide,gboolean flag)
{
	gchar c,*str=NULL;
	gint i=0,j,t;

	t=value;
	if (value==0) {
		str=g_malloc(sizeof(gchar));
		str[i++]=hex[0];
	} else {
		if (flag && value<0) {
			while (t!=0) {
				str=g_realloc(str,(i+1)*sizeof(gchar));
				str[i++]=hex[ABS(t%radix)];
				t/=radix;
			}
		} else {
			while (t!=0) {
				str=g_realloc(str,(i+1)*sizeof(gchar));
				str[i++]=hex[(guint)t%radix];
				t=(guint)t/radix;
			}
		}
	}
	str=g_realloc(str,(i+1)*sizeof(gchar));
	str[i]='\0';
	for (j=0;j<i/2;j++) {
		c=str[j];
		str[j]=str[i-j-1];
		str[i-j-1]=c;
	}
	if (flag && value<0) {
		for (j=i;j>=0;j--)
			str[j+1]=str[j];
		str[0]='-';
		i++;
	}
	if (i<ABS(wide)) {
		str=g_realloc(str,(ABS(wide)+1)*sizeof(gchar));
		for (j=i;j>=0;j--)
			str[j+ABS(wide)-i]=str[j];
		if (wide>0) {
			for (j=0;j<wide-i;j++)
				str[j]=' ';
		} else {
			for (j=0;j<-wide-i;j++)
				str[j]=hex[0];
			if (str[j]=='-') {
				str[0]='-';
				str[j]=hex[0];
			}
		}
	}
	return str;
}


/*	ʸ󢪿
	value,
	  str,ʸ
	radix,
	 flag,TRUE:椢,FALSE:ʤ
	  RET,TRUE:ｪλ,FALSE:顼										*/
gboolean strval(gint *value,gchar *str,gint radix,gboolean flag)
{
	gchar c;
	gint i,j,t;

	*value=0;
	for (i=0;str[i]!='\0' && str[i]==' ';i++);
	if (str[i]=='\0')
		return FALSE;
	if (flag && str[i]=='-') {
		i++;
		while (str[i]!='\0') {
			t=*value;
			*value*=radix;
			c=toupper(str[i]);
			for (j=0;j<radix;j++)
				if (hex[j]==c)
					break;
			*value+=j;
			if (j==radix || *value<t) {
				*value=t;
				return FALSE;
			}
			i++;
		}
		if (*value<0) {
			*value=0;
			return FALSE;
		}
		*value=-*value;
	} else {
		while (str[i]!='\0') {
			t=*value;
			*value*=radix;
			c=toupper(str[i]);
			for (j=0;j<radix;j++)
				if (hex[j]==c)
					break;
			*value+=j;
			if (j==radix || *value<t) {
				*value=t;
				return FALSE;
			}
			i++;
		}
	}
	return TRUE;
}

==> tmaid-2.1.3.tar.gz: tmaid-2.1.3/src/other.c <==
/*
    Text maid
    copyright (c) 1998-2002 Iwamoto,Kazuki http://www.maid.org/ iwm@maid.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <ctype.h>
#include <stdio.h>
#include <gdk/gdkkeysyms.h>
#include "other.h"


/******************************************************************************
*                                                                             *
* åܥåؿ                                                    *
*                                                                             *
******************************************************************************/
static gint result;
static GtkWidget *dialog;


/* ܥ󤬲줿 */
static void message_box_clicked(GtkWidget *widget,gint number)
{
	result=number;
    gtk_widget_destroy(dialog);
}


/* ESC줿 */
static gboolean message_box_dialog_key_press(GtkWidget *widget,
										GdkEventKey *event,gpointer user_data)
{
	if (event->keyval==GDK_Escape)
		gtk_widget_destroy(widget);
	return FALSE;
}


/*	åܥåɽ
	title,ȥ
	 text,ʸ
	  def,ǥեȤΥܥ
	  ...,ܥΥ٥
	  RET,줿ܥ													*/
gint message_box(gchar *title,gchar *text,gint def,...)
{
	va_list ap;
	gchar *name;
	gint i=0;
	GtkWidget *button,*label,*def_button=NULL;

	va_start(ap,def);
	result=-1;
	dialog=gtk_dialog_new();
	gtk_signal_connect_after(GTK_OBJECT(dialog),"key-press-event",
						GTK_SIGNAL_FUNC(message_box_dialog_key_press),NULL);
	gtk_signal_connect(GTK_OBJECT(dialog),"destroy",gtk_main_quit,NULL);
	/* å */
	gtk_window_set_title(GTK_WINDOW(dialog),title);
	label=gtk_label_new(text);
	gtk_misc_set_padding(GTK_MISC(label),8,8);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),label,TRUE,TRUE,0);
	while ((name=va_arg(ap,gchar *))!=NULL) {
		button=gtk_button_new_with_label(name);
		gtk_signal_connect(GTK_OBJECT(button),"clicked",
											message_box_clicked,(gpointer)i);
		GTK_WIDGET_SET_FLAGS(button,GTK_CAN_DEFAULT);
		if (i==def)
			def_button=button;
		gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area),
														button,TRUE,FALSE,8);
		i++;
	}
	va_end(ap);

	/* ɽ */
	if (def_button!=NULL)
		gtk_widget_grab_focus(def_button);

	gtk_grab_add(dialog);
	gtk_widget_show_all(dialog);
	gtk_main();

	return result;
}


/******************************************************************************
*                                                                             *
* ٥ؿ                                                              *
*                                                                             *
******************************************************************************/
/*	ե¸߳ǧ
	file,ե̾
	 RET,TRUE:¸ߤ,FALSE:¸ߤʤ										*/
gboolean isfile(gchar *file)
{
	FILE *fp;

	if ((fp=fopen(file,"r"))!=NULL) {
		fclose(fp);
		return TRUE;
	}
	return FALSE;
}


/*	Сꤹ
	   scroll,륦å
	     func,ʥؿ
	func_data,ǡ
	      min,Ǿ
	      max,
	     page,ڡ
	      pos,															*/
void set_scroll_bar(GtkWidget *scroll,GtkSignalFunc func,gpointer func_data,
										gint min,gint max,gint page,gint pos)
{
	GtkObject *adjust;

	adjust=gtk_adjustment_new(pos,min,max,1,page,page-1);
	gtk_range_set_adjustment(GTK_RANGE(scroll),GTK_ADJUSTMENT(adjust));
	gtk_range_slider_update(GTK_RANGE(scroll));
	if (func!=NULL)
		gtk_signal_connect(GTK_OBJECT(adjust),"value-changed",func,func_data);
}


/*	WindowƤ򥹥뤹
	widget,å
	    dx,Xΰư
	    dy,Yΰư
	     x,뤹ϰϤαXɸ
	     y,뤹ϰϤαYɸ
	 width,뤹ϰϤ
	height,뤹ϰϤι⤵											*/
void scroll_window(GtkWidget *widget,gint dx,gint dy,
										gint x,gint y,gint width,gint height)
{
	GdkGC *gc;
	GdkRectangle rc;

	if (ABS(dx)>width || ABS(dy)>height) {
		rc.x=x;
		rc.y=y;
		rc.width=width;
		rc.height=height;
		gtk_widget_draw(widget,&rc);
	} else if (dx!=0 || dy!=0) {
		gc=gdk_gc_new(widget->window);
		gdk_window_copy_area(widget->window,gc,dx>0?x+dx:x,dy>0?y+dy:y,
										widget->window,dx>0?x:x-dx,dy>0?y:y-dy,
												width-ABS(dx),height-ABS(dy));
		gdk_gc_destroy(gc);
		if (dx!=0) {
			rc.x=dx>0?x:x+width+dx;
			rc.y=y;
			rc.width=ABS(dx);
			rc.height=height;
			gtk_widget_draw(widget,&rc);
		}
		if (dy!=0) {
			rc.x=x;
			rc.y=dy>0?y:y+height+dy;
			rc.width=width;
			rc.height=ABS(dy);
			gtk_widget_draw(widget,&rc);
		}
	}
}


static gchar hex[16]={	'0','1','2','3','4','5','6','7',
						'8','9','A','B','C','D','E','F'};


/*	͢ʸ
	value,
	radix,
	 wide,
	 flag,TRUE:椢,FALSE:ʤ
	  RET,ʸ															*/
gchar *valstr(gint value,gint radix,gint wide,gboolean flag)
{
	gchar c,*str=NULL;
	gint i=0,j,t;

	t=value;
	if (value==0) {
		str=g_malloc(sizeof(gchar));
		str[i++]=hex[0];
	} else {
		if (flag && value<0) {
			while (t!=0) {
				str=g_realloc(str,(i+1)*sizeof(gchar));
				str[i++]=hex[ABS(t%radix)];
				t/=radix;
			}
		} else {
			while (t!=0) {
				str=g_realloc(str,(i+1)*sizeof(gchar));
				str[i++]=hex[(guint)t%radix];
				t=(guint)t/radix;
			}
		}
	}
	str=g_realloc(str,(i+1)*sizeof(gchar));
	str[i]='\0';
	for (j=0;j<i/2;j++) {
		c=str[j];
		str[j]=str[i-j-1];
		str[i-j-1]=c;
	}
	if (flag && value<0) {
		for (j=i;j>=0;j--)
			str[j+1]=str[j];
		str[0]='-';
		i++;
	}
	if (i<ABS(wide)) {
		str=g_realloc(str,(ABS(wide)+1)*sizeof(gchar));
		for (j=i;j>=0;j--)
			str[j+ABS(wide)-i]=str[j];
		if (wide>0) {
			for (j=0;j<wide-i;j++)
				str[j]=' ';
		} else {
			for (j=0;j<-wide-i;j++)
				str[j]=hex[0];
			if (str[j]=='-') {
				str[0]='-';
				str[j]=hex[0];
			}
		}
	}
	return str;
}


/*	ʸ󢪿
	value,
	  str,ʸ
	radix,
	 flag,TRUE:椢,FALSE:ʤ
	  RET,TRUE:ｪλ,FALSE:顼										*/
gboolean strval(gint *value,gchar *str,gint radix,gboolean flag)
{
	gchar c;
	gint i,j,t;

	*value=0;
	for (i=0;str[i]!='\0' && str[i]==' ';i++);
	if (str[i]=='\0')
		return FALSE;
	if (flag && str[i]=='-') {
		i++;
		while (str[i]!='\0') {
			t=*value;
			*value*=radix;
			c=toupper(str[i]);
			for (j=0;j<radix;j++)
				if (hex[j]==c)
					break;
			*value+=j;
			if (j==radix || *value<t) {
				*value=t;
				return FALSE;
			}
			i++;
		}
		if (*value<0) {
			*value=0;
			return FALSE;
		}
		*value=-*value;
	} else {
		while (str[i]!='\0') {
			t=*value;
			*value*=radix;
			c=toupper(str[i]);
			for (j=0;j<radix;j++)
				if (hex[j]==c)
					break;
			*value+=j;
			if (j==radix || *value<t) {
				*value=t;
				return FALSE;
			}
			i++;
		}
	}
	return TRUE;
}
